;; -*- Mode: Lisp; rcs-header: "$Header: /hope/lwhope1-cam/hope.0/compound/33/LISPcapi-examples/RCS/applications:cocoa-application.lisp,v 1.2.7.1 2009/08/21 21:13:00 davef Exp $" -*-

;;----------------------------------------------------------------------------
;;
;; examples/capi/applications/cocoa-application.lisp
;;
;; This example shows how you use application interface objects to
;; control the application menu in CAPI with Cocoa on Mac OS X.
;;
;; There are two example applications defined below:
;;  - A simple one with a single window (i.e. supports one open document).
;;    When run outside the LispWorks IDE, the application quits when
;;    the document window is closed.
;;  - A full one with multiple windows (i.e. supports many open documents).
;;    When run outside the LispWorks IDE, the application remains showing
;;    the application interface's menu bar (the Circle menu) when all
;;    document windows are closed.
;;
;; To try the example, compile and load this file and then execute one of:
;;
;;      (CL-USER::TEST-COCOA-APPLICATION-SIMPLE)
;;      (CL-USER::TEST-COCOA-APPLICATION-FULL)
;;
;; or see the examples in delivery/macos/ for details on how to build
;; them into standalone applications.
;;
;;----------------------------------------------------------------------------
;; Copyright (c) 1987--2010 LispWorks Ltd. All rights reserved.
;;----------------------------------------------------------------------------



(in-package "CL-USER")

;;----------------------------------------------------------------------------
;; The main interface
;;----------------------------------------------------------------------------

(capi:define-interface cocoa-application-test ()
  ((runp :initform nil))
  (:panes
   (label capi:title-pane)
   (drawing capi:output-pane
            :display-callback 'cocoa-application-test-display))
  (:layouts
   (main-layout capi:column-layout
                '(label drawing)))
  (:menus
   (action-menu "Action"
                (("Start Drawing"
                  :callback 'cocoa-application-test-start
                  :callback-type :interface))))
  (:menu-bar action-menu)
  (:default-initargs
   :layout 'main-layout
   :best-width 250
   :best-height 250))

(defvar *cocoa-application-test-count* 0)

(defmethod initialize-instance :after ((self cocoa-application-test) &key)
  (setf (capi:interface-title self)
        (format nil "My Window ~D" (incf *cocoa-application-test-count*))))

(defvar *cocoa-application-test-outer-color* :red)

(defun set-cocoa-application-test-outer-color (color)
  (setq *cocoa-application-test-outer-color* color))

(defun cocoa-application-test-display (pane x y width height)
  (declare (ignore x y width height))
  (when (slot-value (capi:element-interface pane) 'runp)
    (gp:draw-circle pane 100 100 100
                    :foreground *cocoa-application-test-outer-color*)
    (gp:draw-circle pane 100 100 50
                    :foreground :blue)))

(defun cocoa-application-test-start (self)
  (setf (slot-value self 'runp) t)
  (cocoa-application-test-update self))

(defun cocoa-application-test-update (self)
  (gp:invalidate-rectangle (slot-value self 'drawing)))

(defun cocoa-application-test-set-label (self text)
  (setf (capi:title-pane-text (slot-value self 'label)) text))


;;----------------------------------------------------------------------------
;; The simple single-window application
;;----------------------------------------------------------------------------

(capi:define-interface cocoa-application-interface-simple-test (capi:cocoa-default-application-interface)
  ()
  (:default-initargs
   :title "My Application Simple"
   :message-callback 'cocoa-application-interface-simple-test-message))

(defvar *test-cocoa-application-simple* nil)

(defun cocoa-application-interface-simple-test-message (self message &rest args)
  (declare (ignore self))
  (case message
    (:open-file
     (let ((filename (car args)))
       (when (equal (pathname-type filename) "lwtestsimple")
         (when *test-cocoa-application-simple*
           (capi:execute-with-interface *test-cocoa-application-simple*
                                        'cocoa-application-test-set-label
                                        *test-cocoa-application-simple*
                                        filename)))))))

(capi:define-interface cocoa-application-test-simple (cocoa-application-test)
  ()
  (:default-initargs
   :destroy-callback 'cocoa-application-test-quit))

(defun cocoa-application-test-quit (self)
  (when (null (cdr (capi:screen-interfaces ; don't quit if in the LW IDE
                    (capi:element-screen self))))
    (quit)))

(defun test-cocoa-application-simple ()
  (capi:set-application-interface
   (make-instance 'cocoa-application-interface-simple-test))
  ;; Start the application with a single initial window.
  (setq *test-cocoa-application-simple*
        (capi:display (make-instance 'cocoa-application-test-simple))))


;;----------------------------------------------------------------------------
;; The full multiple-window application
;;----------------------------------------------------------------------------

(capi:define-interface cocoa-application-interface-full-test (capi:cocoa-default-application-interface)
  ()
  (:menus
   (application-menu
    "My Application Full"
    ((:component
      (("About My Application Full"
        :callback 'cocoa-application-interface-full-test-about
        :callback-type :none)))
     (:component
      (("Preferences..."
        :callback 'cocoa-application-interface-full-test-preferences
        :callback-type :none)))
     (:component
      ()
      ;; This is a special named component where the CAPI will
      ;; attach the standard Services menu.
      :name :application-services)
     (:component
      (("Hide My Application Full"
        :accelerator "accelerator-h"
        :callback-data :hidden)
       ("Hide Others"
        :accelerator "accelerator-meta-h"
        :callback-data :others-hidden)
       ("Show All"
        :callback-data :all-normal))
      :callback #'(setf capi:top-level-interface-display-state)
      :callback-type :data-interface)
     (:component
      (("Quit My Application Full"
        :accelerator "accelerator-q"
        :callback 'capi:destroy
        :callback-type :interface)))))
   (circle-menu
    "Circle"
    (("New"
      :callback 'cocoa-application-interface-full-test-new
      :callback-type :none))))
  (:menu-bar application-menu circle-menu)
  (:default-initargs
   :title "My Application Full"
   :application-menu 'application-menu
   :message-callback 'cocoa-application-interface-full-test-message
   :dock-menu 'make-test-cocoa-application-full-dock-menu))

(defun cocoa-application-interface-full-test-about ()
  (capi:display-message-on-screen (capi:convert-to-screen nil)
                                  "My Application Full is a CAPI demo"))

(defun cocoa-application-interface-full-test-preferences ()
  (multiple-value-bind (color okp)
      (capi:prompt-for-color "Outer circle color"
                             :color *cocoa-application-test-outer-color*)
    (when okp
      (set-cocoa-application-test-outer-color color)
      (dolist (interface (capi:collect-interfaces 'cocoa-application-test-full))
        (capi:execute-with-interface interface
                                     'cocoa-application-test-update
                                     interface)))))

(defun cocoa-application-interface-full-test-new ()
  (capi:display (make-instance 'cocoa-application-test-full)))

(defun cocoa-application-interface-full-test-message (self message &rest args)
  (declare (ignore self))
  (case message
    (:open-file
     (let ((filename (car args)))
       (when (equal (pathname-type filename) "lwtestfull")
         (let ((interface (cocoa-application-interface-full-test-new)))
           (capi:execute-with-interface interface
                                        'cocoa-application-test-set-label
                                        interface
                                        filename)))))))

(capi:define-menu make-test-cocoa-application-full-dock-menu (self)
  "Dock Menu"
  ((:component
    ()
    :interaction :single-selection
    :items-function #'(lambda (self)
                        (declare (ignore self))
                        (capi:collect-interfaces 'cocoa-application-test-full))
    :selected-item-function #'(lambda (interface)
                                (capi:screen-active-interface
                                 (capi:convert-to-screen interface)))
    :print-function 'capi:interface-title
    :callback-type :data
    :callback 'capi:activate-pane)))

(capi:define-interface cocoa-application-test-full (cocoa-application-test)
  ()
  (:menus
   (circle-menu
    "Circle"
    (("New"
      :callback 'cocoa-application-interface-full-test-new
      :callback-type :none))))
  (:menu-bar circle-menu action-menu))

(defun test-cocoa-application-full ()
  (capi:set-application-interface
   (make-instance 'cocoa-application-interface-full-test))
  ;; Start the application with no windows initially.
  (capi:convert-to-screen nil))

