;; -*- Mode: Lisp; rcs-header: "$Header: /hope/lwhope1-cam/hope.0/compound/33/LISPcapi-examples/RCS/choice:text-input-choice.lisp,v 1.1.9.1 2009/08/21 21:13:02 davef Exp $" -*-

;; Copyright (c) 1987--2010 LispWorks Ltd. All rights reserved.

;;----------------------------------------------------------------------------
;;
;; examples/capi/choice/text-input-choice.lisp
;;
;; This example demonstrates the uses of text-input-choices in the CAPI.
;; To try it, compile and load this file and then execute:
;;
;;      (CL-USER::TEST-TEXT-INPUT-CHOICE)
;;
;;----------------------------------------------------------------------------

(in-package "CL-USER")


;;----------------------------------------------------------------------------
;; Define the interface text-input-choice-test
;;----------------------------------------------------------------------------

(capi:define-interface text-input-choice-test ()
  () 
  (:panes
   (input-string
    capi:text-input-choice
    :reader input-string-pane
    :title "String:"
    :selection-callback 'text-input-choice-show-string
    :callback 'text-input-choice-new-string
    :popup-callback 'text-input-choice-popping-up)
   (output-string
    capi:display-pane
    :reader output-string-pane
    :title "Inverted:"
    :title-position :left
    :visible-max-width nil)
   (invert
    capi:push-button
    :reader invert-pane
    :text "Invert"
    :default-p t
    :selection-callback 'text-input-choice-invert
    :callback-type :interface)
   (random
    capi:check-button
    :text "Random"
    :reader random-pane
    :selected t))
  (:layouts
   (default-layout
    capi:column-layout
    '((capi:row-layout (input-string invert random)
                       :adjust :center)
      output-string)))
  (:default-initargs
   :title "Text Input Choice Test"
   :best-width 500))


;;----------------------------------------------------------------------------
;; A generic callback
;;----------------------------------------------------------------------------

(defun text-input-choice-show-string (string interface)
  (setf (capi:display-pane-text (output-string-pane interface))
        (map 'string
             #'(lambda (char)
                 (if (upper-case-p char)
                     (char-downcase char)
                   (char-upcase char)))
             string)))

(defun text-input-choice-new-string (string interface)
  (text-input-choice-show-string string interface)
  (unless (capi:button-selected (random-pane interface))
    (let ((input-string (input-string-pane interface)))
      (setf (capi:collection-items input-string)
            (adjoin string (coerce (capi:collection-items input-string) 'list)
                    :test 'equal)))))

(defun text-input-choice-invert (interface)
  (text-input-choice-new-string
   (capi:text-input-pane-text
    (input-string-pane interface))
   interface))

(defun text-input-choice-popping-up (pane)
  (let ((interface (capi:element-interface pane)))
    (when (capi:button-selected (random-pane interface))
      (setf (capi:collection-items pane)
            (let* ((names (loop for symbol being the external-symbols of "COMMON-LISP"
                                collect symbol))
                   (Name-count (length names)))
              (loop repeat (+ 2 (random 8))
                    collect (symbol-name (nth (random name-count) names))))))))


;;----------------------------------------------------------------------------
;; A simple test function
;;----------------------------------------------------------------------------

(defun test-text-input-choice ()
  (capi:display (make-instance 'text-input-choice-test)))
