;; -*- Mode: Lisp; rcs-header: "$Header: /hope/lwhope1-cam/hope.0/compound/33/LISPcapi-examples/RCS/editor:change-callback.lisp,v 1.3.7.1 2009/08/21 21:13:00 davef Exp $" -*-

;; Copyright (c) 1987--2010 LispWorks Ltd. All rights reserved.

;;----------------------------------------------------------------------------
;;
;; examples/capi/editor/change-callback.lisp
;;
;; This example demonstrates using the CHANGE-CALLBACK
;;; of an EDITOR-PANE  
;; To try it, compile and load this file and then execute:
;;
;;      (CL-USER::TEST-EDITOR-PANE-CHANGE-CALLBACK)
;;
;; Then type some text into the editor-pane, and when you type
;; a space it colours the last word according to its length. 
;; 
;;---------------------------------

(in-package "CL-USER")


(defvar *faces-array*
  (vector (editor:make-face :red :foreground :red :if-exists t)
          (editor:make-face :blue :foreground :blue :if-exists t)
          (editor:make-face :navyblue :foreground :navyblue :if-exists t)
          (editor:make-face :yellow :foreground :yellow :if-exists t)
          (editor:make-face :green :foreground :green :if-exists t)))

(defun colour-last-word (pane point old-len new-len)
  (declare (ignore pane))
  (when  (and (= old-len 0)
              (= new-len 1) ;;inserted one character
              (eq (editor:character-at point 0) #\space));; which is a space
    (let ((minus-len -1))
      
      ;;; find the length of the sequence of alphanumeric characters backwards
      (loop 
       (let ((char (editor:character-at point minus-len)))
         (unless (and char  ;; char is NIL if beginning of buffer
                      (alphanumericp char))
           (incf minus-len)
           (return)))
       ;; forward over the character were we stopped. 
       (decf minus-len))

      (editor:with-point ((start point))
        (editor:character-offset start minus-len) ;; start to the beginning of the word
        (editor:put-text-property  ;; colour the word
         start point 
         'editor:face (svref *faces-array*
                             (let ((len (- minus-len)))
                               (if (> len 14)
                                   4
                                 (truncate len 3)))))))))
                                        

(defun test-editor-pane-change-callback ()
  (capi:contain (make-instance 
                 'capi:editor-pane
                 :change-callback 'colour-last-word
                 :buffer (format nil "~a" (gensym "new buffer ")) ;;unique buffer
                 :flag :output  ;;; don't bother to save
                 )))        
