;; -*- rcs-header: "$Header: /hope/lwhope1-cam/hope.0/compound/33/LISPcapi-examples/RCS/elements:text-input-choice.lisp,v 1.1.1.1 2009/08/21 21:13:01 davef Exp $" -*-

;;----------------------------------------------------------------------------
;;
;; examples/capi/elements/text-input-choice.lisp
;;
;; This example demonstrates the uses of text-input-choice in the CAPI.
;; To try it, compile and load this file and then execute:
;;
;;      (CL-USER::TEST-TEXT-INPUT-CHOICE)
;;
;;----------------------------------------------------------------------------
;; Copyright (c) 1987--2010 LispWorks Ltd. All rights reserved.
;;----------------------------------------------------------------------------


(in-package "CL-USER")


(defvar *max-chars* 20)

(capi:define-interface text-input-choice-test ()
  () ;
  (:panes
   (text-input-choice
    capi:text-input-choice
    :accessor text-input-choice
    :items '("Item 0" "Item 1" "Item 2")
    :change-callback 'text-input-choice-test-change-callback
    :max-characters *max-chars*
    :callback-type :interface-data
    :callback 'text-input-choice-test-callback
    :selection-callback 'text-input-choice-test-selection-callback)
   (selection-pane
    capi:text-input-pane
    :accessor selection-pane
    :text "0"
    :callback-type :interface-data
    :callback 'text-input-choice-set-selection)
   (text-pane
    capi:display-pane
    :accessor text-pane
    :min-height '(character 1)
    :max-width nil)
   (caret-pane
    capi:display-pane
    :accessor caret-pane
    :min-height '(character 1)
    :max-width nil)
   (max-characters-pane
    capi:text-input-pane
    :text (princ-to-string *max-chars*)
    #-:Win32 :confirm-change-function #-:Win32 'validate-choice-text ; not supported on Windows
    :callback-type :interface-data
    :callback 'set-choice-max-characters)
   (disable-button
    capi:radio-button-panel
    :title "Enabled"
    :title-position :left
    :items '(:yes :no :read-only)
    :print-function 'string-capitalize
    :selected-item :yes
    :callback-type :interface-data
    :selection-callback 'update-text-input-choice-state)
   (randomize-button
    capi:push-button
    :text "Randomize"
    :callback 'randomize-text-input-choice-test
    :callback-type :interface))
  (:layouts
   (text-pane-layout
    capi:grid-layout
    '("Enter text:" text-input-choice
      "Selection:" selection-pane
      "Text:" text-pane
      "Caret:" caret-pane
      "Max Chars:" max-characters-pane)
    :y-adjust :center)
   (callback-layout
    capi:column-layout
    '(text-pane-layout disable-button randomize-button)
    :title "Callbacks"
    :title-position :frame)
   (main-layout
    capi:column-layout
    '(callback-layout)))
  (:default-initargs
   :title "Text Input Pane Test"
   :best-width 300
   :layout 'main-layout))


;;----------------------------------------------------------------------------
;; Callbacks
;;----------------------------------------------------------------------------

(defun text-input-choice-test-callback (interface text)
  (declare (ignore interface))
  (capi:display-message "~S" text))

(defun text-input-choice-test-change-callback (text pane interface caret-position)
  (declare (ignore pane))
  (setf (capi:display-pane-text (caret-pane interface))
        (princ-to-string caret-position))
  (setf (capi:display-pane-text (text-pane interface))
        text))

(defun text-input-choice-test-selection-callback (interface text)
  (let ((selection (position text (capi:collection-items
                                   (text-input-choice interface))
                             :test 'equal)))
    (setf (capi:text-input-pane-text (selection-pane interface))
          (if selection
              (prin1-to-string selection)
            ""))))

(defun update-text-input-choice-state (self enabled)
  (setf (capi:text-input-pane-enabled (text-input-choice self))
        (case enabled
          (:yes t)
          (:no nil)
          (otherwise :read-only))))

(defun text-input-choice-set-selection (self selection-text)
  (let ((index (parse-integer selection-text :junk-allowed t))
        (choice (text-input-choice self)))
    (if (and index
             (>= (capi:count-collection-items choice) index 0))
        (setf (capi:choice-selection choice) index)
      (capi:display-message "Not a valid index (must be integer in the inclusive ange 0-2: ~a" selection-text))))

#-:Win32
(defun validate-choice-text (text pane interface caret-position)
  (declare (ignore pane interface caret-position))
  (multiple-value-bind
      (integer length)
      (parse-integer text :junk-allowed t)
    (declare (ignore integer))
    (eq length (length text))))

(defun set-choice-max-characters (self text)
  (let ((value (parse-integer text :junk-allowed t))
        (pane (text-input-choice self)))
    (if (integerp value)
        (setf (capi:text-input-pane-max-characters pane) value)
      (capi:display-message "Non-integer for max-characters: ~S" text))))

(defun randomize-text-input-choice-test (self)
  (let ((pane (text-input-choice self)))
    (if (zerop (random 2))
        (setf (capi:text-input-pane-text pane)
              (with-output-to-string (text)
                (loop repeat (+ 2 (random 8))
                      do
                      (write-char (code-char (+ (char-code #\a)
                                                (random 26)))
                                  text))))
      (setf (capi:text-input-pane-caret-position pane)
            (random (length (capi:text-input-pane-text pane)))))))


;;----------------------------------------------------------------------------
;; test-text-input-choice
;;----------------------------------------------------------------------------

(defun test-text-input-choice ()
  (capi:display (make-instance 'text-input-choice-test)))
