;; -*- rcs-header: "$Header: /hope/lwhope1-cam/hope.0/compound/33/LISPcapi-examples/RCS/graphics:pixmap-port.lisp,v 1.2.11.1 2009/08/21 21:13:02 davef Exp $" -*-

;;----------------------------------------------------------------------------
;;
;; examples/capi/graphics/pixmap-port.lisp
;;
;; This example demonstrates the uses of pixmap-port within a
;; pinboard-layouts in the CAPI.
;;
;; To try it, compile and load this file and then execute:
;;
;;      (CL-USER::TEST-REDRAWING-WITH-PIXMAP)
;;
;;----------------------------------------------------------------------------
;; Copyright (c) 1987--2010 LispWorks Ltd. All rights reserved.
;;----------------------------------------------------------------------------


(in-package "CL-USER")


;;----------------------------------------------------------------------------
;; Define an interface
;;----------------------------------------------------------------------------

(capi:define-interface redrawing-with-pixmap ()
  ((pixmap :initform nil))
  (:panes
   (pixmap-object capi:drawn-pinboard-object
                  :display-callback 'draw-interface-pixmap))
  (:layouts
   (pixmap-layout capi:pinboard-layout
                  '((pixmap-object :x 5 :y 5 :width 50 :height 50))))
  (:default-initargs
   :destroy-callback 'destroy-interface-pixmap))

(defun draw-interface-pixmap (pinboard object x y width height)
  ;; x,y,width & height are the coords & size in pinboard of the object to be drawn.
  (with-slots (pixmap) (capi:element-interface pinboard)
    (capi:with-geometry object
      (unless pixmap
        (setf pixmap (compute-interface-pixmap pinboard)))
      (gp:pixblt pinboard boole-1 pixmap
                 x y width height
                 (- x capi:%x%)
                 (- y capi:%y%)))))


;;----------------------------------------------------------------------------
;; Code to create and destroy the pixmap.
;;----------------------------------------------------------------------------

(defun compute-interface-pixmap (pinboard)
  (let ((pixmap (gp:create-pixmap-port pinboard 50 50 :clear t :background :white)))
    (setf (gp:graphics-port-foreground pixmap) :black)
    (dotimes (y 50)
      (dotimes (x 50)
        (when (zerop (random 2))
          (gp:draw-point pixmap x y))))
    (gp:draw-circle pixmap 25 25 20 :foreground :red :thickness 3)
    pixmap))

(defun destroy-interface-pixmap (interface)
  (with-slots (pixmap) interface
    (when pixmap
      (gp:destroy-pixmap-port pixmap))))

;;----------------------------------------------------------------------------
;; The test function
;;----------------------------------------------------------------------------

(defun test-redrawing-with-pixmap ()
  (capi:display (make-instance 'redrawing-with-pixmap
                               :best-width 100
                               :best-height 100)))

