;; -*- Mode: Lisp; rcs-header: "$Header: /hope/lwhope1-cam/hope.0/compound/33/LISPcapi-examples/RCS/output-panes:drawing.lisp,v 1.2.9.1 2009/08/21 21:13:02 davef Exp $" -*-

;;----------------------------------------------------------------------------
;;
;; examples/capi/output-panes/drawing.lisp
;;
;; This example demonstrates the input model of an output pane
;; with a simple redisplay callback.
;; It also demostrates the use of an image on the clipboard (not supported
;; on Motif).
;;
;; To try it, compile and load this file and then execute:
;;
;;      (CL-USER::TEST-DRAWING)
;;
;;----------------------------------------------------------------------------
;; Copyright (c) 1987--2010 LispWorks Ltd. All rights reserved.
;;----------------------------------------------------------------------------

(in-package "CL-USER")

(capi:define-interface drawing-test ()
  ()
  (:panes
   (drawing-pane drawing-pane))
  (:menus
   (file-menu "File"
              (("Exit" :callback 'capi:quit-interface))
              :callback-type :interface)
   (edit-menu "Edit"
              (("Copy" :callback 'drawing-test-copy)
               ("Paste" :callback 'drawing-test-paste
                        :enabled-function 'drawing-test-paste-p))
              :callback-type :interface))
  (:menu-bar file-menu edit-menu)
  (:default-initargs
   :title "Drawing Test"
   :best-width 200
   :best-height 200))

(defun drawing-test-copy (self)
  (with-slots (drawing-pane) self
    (let ((image (gp:make-image-from-port drawing-pane)))
      (capi:set-clipboard drawing-pane nil nil (list :image image
                                                     :string "An image")))))

(defun drawing-test-paste (self)
  (with-slots (drawing-pane) self
    (let ((image (capi:clipboard drawing-pane :image)))
      (set-drawing-background drawing-pane image))))

(defun drawing-test-paste-p (self)
  (with-slots (drawing-pane) self
    (not (capi:clipboard-empty drawing-pane :image))))


(defvar *input-models* 
  '(((:button-1 :press) start-drawing)
    ((:button-1 :motion) continue-drawing)
    ((:button-1 :release) end-drawing)
    ((:button-3 :release) clear-drawing)))

(defclass drawing-pane (capi:output-pane)
  ((background-image :initform nil)
   (complete-lines :initform nil)
   (current-line :initform nil)
   (current-point :initform nil))
  (:default-initargs 
   :display-callback 'repair-drawing
   :input-model *input-models*))

(defmethod start-drawing ((self drawing-pane) x y)
  (with-slots (current-point) self
    (setf current-point (cons x y))))

(defmethod continue-drawing ((self drawing-pane) x y)
  (with-slots (current-line current-point) self
    (when current-point
      (let ((old-x (car current-point))
            (old-y (cdr current-point)))
        (gp:draw-line self old-x old-y x y)
        (setf current-line `(,old-x ,old-y ,x ,y ,@current-line))))
    (setf current-point (cons x y))))

(defmethod end-drawing ((self drawing-pane) x y)
  (declare (ignore x y))
  (with-slots (complete-lines current-line current-point) self
    (push current-line complete-lines)
    (setf current-line nil
          current-point nil)))

(defmethod set-drawing-background ((self drawing-pane) image)
  (with-slots (background-image) self
    (when background-image
      (gp:free-image self background-image))
    (setf background-image image)
    (gp:invalidate-rectangle self)))


;; this callback is "simple" in that it does not limit itself to the
;; "dirty" region

(defmethod repair-drawing ((self drawing-pane) x y width height)
  (declare (ignore x y width height))
  (with-slots (background-image current-line complete-lines) self
    (when background-image
      (gp:draw-image self background-image 0 0))
    (gp:draw-lines self current-line)
    (dolist (line complete-lines)
      (gp:draw-lines self line))))

(defmethod clear-drawing ((self drawing-pane) x y)
  (declare (ignore x y))
  (with-slots (background-image complete-lines current-line current-point) self 
    (when background-image
      (gp:free-image self background-image)
      (setf background-image nil))
    (setf current-line nil
          current-point nil
          complete-lines nil))
  (gp:clear-graphics-port self))


(defun test-drawing ()
  (capi:display (make-instance 'drawing-test)))

