;; -*- Mode: Lisp; rcs-header: "$Header: /hope/lwhope1-cam/hope.0/compound/33/LISPcapi-examples/RCS/output-panes:spirograph.lisp,v 1.3.11.1 2009/08/21 21:13:02 davef Exp $" -*-

;;----------------------------------------------------------------------------
;;
;; examples/capi/output-panes/spriograph.lisp
;;
;; This example demonstrates the input model of an output pane
;; To try it, compile and load this file and then execute:
;;
;;      (CL-USER::TEST-SPIROGRAPH)
;;
;;----------------------------------------------------------------------------
;; Copyright (c) 1987--2010 LispWorks Ltd. All rights reserved.
;;----------------------------------------------------------------------------

(in-package "CL-USER")

;; Create a circular list of colors
(defvar *colors* (let ((a '(:red :green :blue :yellow :black :white)))
                   (setf (cdr (last a)) a)
                   a))

(defun collect-point (port x y)
  (unless (equal (cons x y) (car (capi:capi-object-property port :points)))
    (push (cons x y) (capi:capi-object-property port :points))
    (gp:draw-point port x y)))


(defun finish-with-points (port x y)
  (with-slots (colors) 
      (capi:element-interface port)
    (collect-point port x y)
    (let ((points (capi:capi-object-property port :points)))
      (loop for (point . rest) on points
            for (from-x . from-y) = point
            do 
            (loop for (to-x . to-y) in rest
                  do (gp:draw-line port from-x from-y
                                   to-x to-y
                                   :foreground (car colors)))))
    (setf colors (rest colors))
    (capi:remove-capi-object-property port :points)))

(capi:define-interface spirograph () 
  ((colors :initform *colors*))
  (:panes 
   (output capi:output-pane
           :input-model '(((:button-1 :press) collect-point)
                          ((:button-1 :motion) collect-point)
                          ((:button-1 :release) finish-with-points)))))

(defun test-spirograph ()
  (capi:display
   (make-instance 'spirograph :best-width 500 :best-height 500)))


